/**
 * @package css-modules-types-loader
 * @license MIT
 * @version 0.3.31
 * @author nuintun <nuintun@qq.com>
 * @description A webpack loader that dynamically generates TypeScript typings for CSS modules from css-loader.
 * @see https://github.com/nuintun/css-modules-types-loader#readme
 */

'use strict';

const fs = require('fs');
const os = require('os');
const acorn = require('acorn');
const util = require('util');
const acornWalk = require('acorn-walk');

function _interopDefault(e) {
  return e && e.__esModule ? e : { default: e };
}

const fs__default = /*#__PURE__*/ _interopDefault(fs);

/**
 * @module utils
 */
const rm = util.promisify(fs__default.default.rm);
const writeFile = util.promisify(fs__default.default.writeFile);
/**
 * @function removeFile
 * @description Removes a file from the filesystem if it exists.
 * @param path The path to the file.
 */
async function removeFile(path) {
  if (fs__default.default.existsSync(path)) {
    await rm(path);
  }
}
/**
 * @function isString
 * @description Checks if a value is a string.
 * @param value The value to check.
 */
function isString(value) {
  return Object.prototype.toString.call(value) === '[object String]';
}
/**
 * @function collect
 * @description Collects styles from the given nodes and adds them to the styles array.
 * @param styles The array to which the collected styles will be added.
 * @param left The left node.
 * @param right The right node.
 */
function collect(styles, left, right) {
  if (right.type === 'Literal') {
    const value = right.value;
    if (isString(value)) {
      let key;
      switch (left.type) {
        case 'Identifier':
          key = left.name;
          break;
        case 'Literal':
          key = left.value;
          break;
      }
      if (isString(key)) {
        styles.push([key, value]);
      }
    }
  }
}
/**
 * @function parseStyles
 * @description Parses the styles from the given content.
 * @param content The content to parse the styles from.
 */
function parseStyles(content) {
  let named = false;
  const styles = [];
  const ast = acorn.parse(content, {
    sourceType: 'module',
    ecmaVersion: 'latest'
  });
  acornWalk.simple(ast, {
    ExpressionStatement(node) {
      acornWalk.simple(node, {
        AssignmentExpression({ left }) {
          if (left.type === 'MemberExpression') {
            const { property } = left;
            if (property.type === 'Identifier') {
              if (property.name === 'locals') {
                acornWalk.simple(node, {
                  Property({ key, value }) {
                    collect(styles, key, value);
                  }
                });
              }
            }
          }
        }
      });
    },
    ExportNamedDeclaration(node) {
      named = true;
      acornWalk.simple(node, {
        VariableDeclarator({ id, init }) {
          if (init) {
            collect(styles, id, init);
          }
        }
      });
    }
  });
  return [styles, named];
}
/**
 * @function generateTypings
 * @description Generate typings for CSS modules.
 * @param content The CSS content.
 * @param banner Optional banner string.
 * @param eol End of line character.
 */
function generateTypings(content, banner, eol = os.EOL) {
  const [styles, named] = parseStyles(content);
  if (styles.length > 0) {
    const typings = banner ? [banner, ''] : [];
    if (named) {
      for (const [key, value] of styles) {
        typings.push(`export const ${key}: ${JSON.stringify(value)};`);
      }
    } else {
      typings.push('declare const styles: {');
      for (const [key, value] of styles) {
        typings.push(`  ${key}: ${JSON.stringify(value)}`);
      }
      typings.push('};', '', 'export default styles;');
    }
    return typings.join(eol);
  }
  return null;
}

exports.collect = collect;
exports.generateTypings = generateTypings;
exports.isString = isString;
exports.parseStyles = parseStyles;
exports.removeFile = removeFile;
exports.rm = rm;
exports.writeFile = writeFile;
